package com.huimv.admin.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Pair;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.admin.common.token.TokenSign;
import com.huimv.admin.common.utils.DataUill;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.common.utils.ResultCode;
import com.huimv.admin.entity.BasePigpen;
import com.huimv.admin.entity.EnergyData;
import com.huimv.admin.entity.EnergyEnvDevice;
import com.huimv.admin.entity.EnergyWarningInfo;
import com.huimv.admin.entity.vo.*;
import com.huimv.admin.mapper.BasePigpenMapper;
import com.huimv.admin.mapper.EnergyDataMapper;
import com.huimv.admin.mapper.EnergyEnvDeviceMapper;
import com.huimv.admin.mapper.EnergyWarningInfoMapper;
import com.huimv.admin.service.IEnergyDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 耗能数据 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-02-14
 */
@Service
public class EnergyDataServiceImpl extends ServiceImpl<EnergyDataMapper, EnergyData> implements IEnergyDataService {

    @Autowired
    private EnergyDataMapper dataMapper;
    @Autowired
    private BasePigpenMapper basePigpenMapper;
    @Autowired
    private EnergyWarningInfoMapper warningInfoMapper;
    @Autowired
    private EnergyEnvDeviceMapper envDeviceMapper;

    @Override
    public Result listCount(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<EnergyData> queryWrapper = new QueryWrapper<>();
        Date monthmorning = DataUill.getTimesMonthmorning();
        queryWrapper.eq("farm_id", farmId).ge("create_date", monthmorning);
        EnergyDataVo energyDataVo = dataMapper.listCount(queryWrapper);
        QueryWrapper<EnergyWarningInfo> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId);
        Integer count = warningInfoMapper.selectCount(queryWrapper1);
//        Float f = 0.00f;
//        f = (Float.parseFloat(energyDataVo.getElectricityValue()) +
//                Float.parseFloat(energyDataVo.getFeedValue()) +
//                Float.parseFloat(energyDataVo.getGasValue()) + Float.parseFloat(energyDataVo.getWaterValue()));
//        DecimalFormat df1 = new DecimalFormat("#.00");
        energyDataVo.setCount(count);
        return new Result(ResultCode.SUCCESS, energyDataVo);
    }

    @Override
    public Result listData(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String dataType = paramsMap.get("dataType");
        String timeType = paramsMap.get("timeType");
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");

        if (dataType == null || dataType == "") {
            dataType = "1";
        }
        if (timeType == null || timeType == "") {
            timeType = "3";
        }

        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
        dataQueryWrapper.eq("farm_id", farmId);
        //用水
        if ("1".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                dataQueryWrapper.between("create_date", startTime, endTime);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayWater(dataQueryWrapper));
            }
            //全年
            if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                dataQueryWrapper.ge("create_date", dateTime);
                List<EnergyAllDataVo> energyWaterDataVos = dataMapper.listYearWater(dataQueryWrapper);
                return new Result(ResultCode.SUCCESS, energyWaterDataVos);
            }
            //本月
            if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayWater(dataQueryWrapper));
            }
            //本周
            if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                dataQueryWrapper.ge("create_date", calendar.getTime());
                List<EnergyAllDataVo> energyWaterDataVos = dataMapper.listDayWater(dataQueryWrapper);
                return new Result(ResultCode.SUCCESS, energyWaterDataVos);
            }
        }
        //用电
        else if ("2".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                dataQueryWrapper.between("create_date", startTime, endTime);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayElectricity(dataQueryWrapper));
            }
            //全年
            if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                dataQueryWrapper.ge("create_date", dateTime);
                List<EnergyAllDataVo> energyElectricityDataVos = dataMapper.listYearElectricity(dataQueryWrapper);
                return new Result(ResultCode.SUCCESS, energyElectricityDataVos);
            }
            //本月
            if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayElectricity(dataQueryWrapper));
            }
            //本周
            if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                dataQueryWrapper.ge("create_date", calendar.getTime());
                List<EnergyAllDataVo> energyElectricityDataVos = dataMapper.listDayElectricity(dataQueryWrapper);
                return new Result(ResultCode.SUCCESS, energyElectricityDataVos);
            }
        }
        //饲料
        else if ("3".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                dataQueryWrapper.between("create_date", startTime, endTime);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayFeed(dataQueryWrapper));
            }
            //全年
            if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                dataQueryWrapper.ge("create_date", dateTime);
                List<EnergyAllDataVo> energyFeedDataVos = dataMapper.listYearFeed(dataQueryWrapper);
                return new Result(ResultCode.SUCCESS, energyFeedDataVos);
            }
            //本月
            if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayFeed(dataQueryWrapper));
            }
            //本周
            if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                dataQueryWrapper.ge("create_date", calendar.getTime());
                List<EnergyAllDataVo> energyFeedDataVos = dataMapper.listDayFeed(dataQueryWrapper);
                return new Result(ResultCode.SUCCESS, energyFeedDataVos);
            }
        }
        //天然气
        else if ("4".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                dataQueryWrapper.between("create_date", startTime, endTime);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayGas(dataQueryWrapper));
            }
            //全年
            if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                dataQueryWrapper.ge("create_date", dateTime);
                List<EnergyAllDataVo> energyGasDataVos = dataMapper.listYearGas(dataQueryWrapper);
                return new Result(ResultCode.SUCCESS, energyGasDataVos);
            }
            //本月
            if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayGas(dataQueryWrapper));
            }
            //本周
            if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                dataQueryWrapper.ge("create_date", calendar.getTime());
                List<EnergyAllDataVo> energyGasDataVos = dataMapper.listDayGas(dataQueryWrapper);
                return new Result(ResultCode.SUCCESS, energyGasDataVos);
            }
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listRank(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String dataType = paramsMap.get("dataType");
        String timeType = paramsMap.get("timeType");
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");
        if (dataType == null || dataType == "") {
            dataType = "1";
        }
        if (timeType == null || timeType == "") {
            timeType = "3";
        }
        JSONArray jsonArray = new JSONArray();

        QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("f_type", 2);
        List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
        for (BasePigpen pigpen : pigpens) {
            Float value = 0.00f;
            JSONObject jsonObject = new JSONObject();
            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.like("other2", pigpen.getId()).eq("farm_id", farmId).ne("id", pigpen.getId());
            List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);
            List<Integer> idList = new ArrayList<>();
            basePigpens1.forEach(item -> {
                idList.add(item.getId());
            });//某一层所有单元的id
            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
            if (idList.size() != 0) {
                dataQueryWrapper.eq("farm_id", farmId).in("unit_id", idList);
                if ("4".equals(timeType)) {
                    //自定义
                    startTime = startTime + " 00:00:00";
                    endTime = endTime + " 23:59:59";
                    dataQueryWrapper.between("create_date", startTime, endTime);
                } else if ("3".equals(timeType)) {
                    //本年
                    DateTime dateTime = DateUtil.beginOfYear(new Date());
                    dataQueryWrapper.ge("create_date", dateTime);
                } else if ("2".equals(timeType)) {
                    //本月
                    Date timesMonthmorning = DataUill.getTimesMonthmorning();
                    dataQueryWrapper.ge("create_date", timesMonthmorning);
                } else if ("1".equals(timeType)) {
                    //本周
                    Calendar calendar = Calendar.getInstance();
                    calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                    dataQueryWrapper.ge("create_date", calendar.getTime());
                }
                dataQueryWrapper.select(
                        " IFNULL(SUM(water_value),0) waterValue,IFNULL(SUM(gas_value),0) gasValue" +
                                ",IFNULL(SUM(electricity_value),0) electricityValue" +
                                ",IFNULL(SUM(feed_value),0) feedValue  "
                );
                EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                if ("1".equals(dataType)) {
                    value = Float.parseFloat(energyData.getWaterValue());
                    jsonObject.put("value", value);
                }
                if ("2".equals(dataType)) {
                    value = Float.parseFloat(energyData.getElectricityValue());
                    jsonObject.put("value", value);
                }
                if ("3".equals(dataType)) {
                    value = Float.parseFloat(energyData.getFeedValue());
                    jsonObject.put("value", value);

                }
                if ("4".equals(dataType)) {
                    value = Float.parseFloat(energyData.getGasValue());
                    jsonObject.put("value", value);
                }
                jsonObject.put("buildName", pigpen.getBuildName());

            } else {
                jsonObject.put("value", 0);
                jsonObject.put("buildName", pigpen.getBuildName());
            }
            jsonArray.add(jsonObject);
        }

        JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
        return new Result(ResultCode.SUCCESS, arraySort);
    }

    @Override
    public Result listDataScreen(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<EnergyData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<EnergyAllDataVo> energyGasDataVos = dataMapper.listDayGas(queryWrapper);
        List<EnergyAllDataVo> energyFeedDataVos = dataMapper.listDayFeed(queryWrapper);
        List<EnergyAllDataVo> energyWaterDataVos = dataMapper.listDayWater(queryWrapper);
        List<EnergyAllDataVo> energyElectricityDataVos = dataMapper.listDayElectricity(queryWrapper);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("gas", energyGasDataVos);
        jsonObject.put("water", energyWaterDataVos);
        jsonObject.put("energyElectricity", energyElectricityDataVos);
        jsonObject.put("feed", energyFeedDataVos);
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result listRankScreen(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String dataType = paramsMap.get("dataType");
        if (dataType == null || dataType == "") {
            dataType = "1";
        }
        JSONArray jsonArray = new JSONArray();

        QueryWrapper<BasePigpen> basePigpenQueryWrapper = new QueryWrapper<>();
        basePigpenQueryWrapper.eq("farm_id", farmId).eq("f_type", 2);
        List<BasePigpen> basePigpenList = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的楼层
        Float value = 0.00f;
        for (BasePigpen basePigpen : basePigpenList) {
            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.like("other2", basePigpen.getId()).eq("farm_id", farmId).ne("id", basePigpen.getId());
            List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);
            List<Integer> idList = new ArrayList<>();
            basePigpens1.forEach(item -> {
                idList.add(item.getId());
            });//某一层所有单元的id

            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
            Date timesMonthmorning = DataUill.getTimesMonthmorning();
            JSONObject jsonObject = new JSONObject();
            if (idList.size() != 0) {
                dataQueryWrapper.eq("farm_id", farmId).in("unit_id", idList);
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                dataQueryWrapper.select(
                        "IFNULL(SUM(water_value),0) waterValue,IFNULL(SUM(gas_value),0) gasValue" +
                                ",IFNULL(SUM(electricity_value),0) electricityValue" +
                                ",IFNULL(SUM(feed_value),0) feedValue  "
                );
                EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                if ("1".equals(dataType)) {
                    value = Float.parseFloat(energyData.getWaterValue());
                }
                if ("2".equals(dataType)) {
                    value = Float.parseFloat(energyData.getElectricityValue());
                }
                if ("3".equals(dataType)) {
                    value = Float.parseFloat(energyData.getFeedValue());
                }
                if ("4".equals(dataType)) {
                    value = Float.parseFloat(energyData.getGasValue());
                }
                jsonObject.put("value", value);
                jsonObject.put("buildName", basePigpen.getBuildName());
                value = 0.00f;
            } else {
                jsonObject.put("value", 0);
                jsonObject.put("buildName", basePigpen.getBuildName());
            }
            jsonArray.add(jsonObject);
        }
        JSONArray arraySort = jsonArraySort1(jsonArray, "value", true);
        JSONArray array = new JSONArray();
        if (arraySort.size() < 5) {
            return new Result(ResultCode.SUCCESS, arraySort);
        } else {
            for (int i = 0; i < 5; i++) {
                array.add(arraySort.get(i));
            }
            return new Result(ResultCode.SUCCESS, array);
        }

    }

    @Override
    public Result listDataAll(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {

        String farmId = TokenSign.getFarmIds(httpServletRequest);
        if (farmId.length() > 2) {
            farmId = paramsMap.get("farmId");
        }
        String type = paramsMap.get("type");// 1水 2电
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<EnergyEnvDevice> envDevices = envDeviceMapper.selectList(queryWrapper);

        Date timesmorning = DataUill.getTimesmorning();
        DateTime dateTime = DateUtil.beginOfWeek(new Date());//本周
        Date timesMonthmorning = DataUill.getTimesMonthmorning();//本月
        Date dayStart = (Date) getYesterdayRange().get("startTime");
        Date dayEnd = (Date) getYesterdayRange().get("stopTime");
        Date weekStart = (Date) getLastWeekRange().get("startTime");
        Date weekEnd = (Date) getLastWeekRange().get("stopTime");
        Date monthStart = (Date) getLastMonthRange().get("startTime");
        Date monthEnd = (Date) getLastMonthRange().get("stopTime");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");


        JSONArray jsonArray = new JSONArray();
        for (EnergyEnvDevice envDevice : envDevices) {
            EnergyDataScreenVo dataScreenVo = dataMapper.listDataAll(sdf.format(timesmorning), sdf.format(dateTime), sdf.format(timesMonthmorning),
                    sdf.format(dayStart), sdf.format(dayEnd)
                    , sdf.format(weekStart), sdf.format(weekEnd)
                    , sdf.format(monthStart), sdf.format(monthEnd)
                    , envDevice.getUnitId(), farmId);
            if ("1".equals(type) && "1".equals(envDevice.getRemark())) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("deviceCode", envDevice.getDeviceCode());
                jsonObject.put("buildName", envDevice.getUnitName());
                jsonObject.put("day1", dataScreenVo.getWaterValueToday());
                jsonObject.put("day2", dataScreenVo.getWaterValueTodayUsage());
                jsonObject.put("day3", dataScreenVo.getWaterValueYesterdayUsage());
                jsonObject.put("week1", dataScreenVo.getWaterValueThisWeekUsage());
                jsonObject.put("week2", dataScreenVo.getWaterValueLastWeekUsage());
                jsonObject.put("month1", dataScreenVo.getElectricityValueThisMonthUsage());
                jsonObject.put("month2", dataScreenVo.getElectricityValueLastMonthUsage());
                double day2 = Double.parseDouble(dataScreenVo.getWaterValueTodayUsage());
                double day3 = Double.parseDouble(dataScreenVo.getWaterValueYesterdayUsage());
                double week1 = Double.parseDouble(dataScreenVo.getWaterValueThisWeekUsage());
                double week2 = Double.parseDouble(dataScreenVo.getWaterValueLastWeekUsage());
                double month1 = Double.parseDouble(dataScreenVo.getElectricityValueThisMonthUsage());
                double month2 = Double.parseDouble(dataScreenVo.getElectricityValueLastMonthUsage());
                DecimalFormat def = new DecimalFormat("0.00");
                String day4 = def.format(day2 - day3);
                String week3 = def.format(week1 - week2);
                String month3 = def.format(month1 - month2);
                jsonObject.put("day4", day4);
                jsonObject.put("week3", week3);
                jsonObject.put("month3", month3);
                jsonArray.add(jsonObject);
            }
            if ("2".equals(type) && "0".equals(envDevice.getRemark())) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("deviceCode", envDevice.getDeviceCode());
                jsonObject.put("buildName", envDevice.getUnitName());
                jsonObject.put("day1", dataScreenVo.getElectricityValueToday());
                jsonObject.put("day2", dataScreenVo.getElectricityValueTodayUsage());
                jsonObject.put("day3", dataScreenVo.getElectricityValueYesterdayUsage());
                jsonObject.put("week1", dataScreenVo.getElectricityValueThisWeekUsage());
                jsonObject.put("week2", dataScreenVo.getElectricityValueLastWeekUsage());
                jsonObject.put("month1", dataScreenVo.getElectricityValueThisMonthUsage());
                jsonObject.put("month2", dataScreenVo.getElectricityValueLastMonthUsage());
                double day2 = Double.parseDouble(dataScreenVo.getElectricityValueTodayUsage());
                double day3 = Double.parseDouble(dataScreenVo.getElectricityValueYesterdayUsage());
                double week1 = Double.parseDouble(dataScreenVo.getElectricityValueThisWeekUsage());
                double week2 = Double.parseDouble(dataScreenVo.getElectricityValueLastWeekUsage());
                double month1 = Double.parseDouble(dataScreenVo.getElectricityValueThisMonthUsage());
                double month2 = Double.parseDouble(dataScreenVo.getElectricityValueLastMonthUsage());
                DecimalFormat def = new DecimalFormat("0.00");
                String day4 = def.format(day2 - day3);
                String week3 = def.format(week1 - week2);
                String month3 = def.format(month1 - month2);
                jsonObject.put("day4", day4);
                jsonObject.put("week3", week3);
                jsonObject.put("month3", month3);
                jsonArray.add(jsonObject);
            }

        }
        return new Result(ResultCode.SUCCESS, jsonArray);
    }


    @Override
    public Result listDataAll2(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {

        String farmId = TokenSign.getFarmIds(httpServletRequest);
        if (farmId.length() > 2) {
            farmId = paramsMap.get("farmId");
        }
        String type = paramsMap.get("type");// 1水 2电
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<EnergyEnvDevice> envDevices = envDeviceMapper.selectList(queryWrapper);

        Date timesmorning = DataUill.getTimesmorning();
        DateTime dateTime = DateUtil.beginOfWeek(new Date());//本周
        Date timesMonthmorning = DataUill.getTimesMonthmorning();//本月
        Date dayStart = (Date) getYesterdayRange().get("startTime");
        Date dayEnd = (Date) getYesterdayRange().get("stopTime");
        Date weekStart = (Date) getLastWeekRange().get("startTime");
        Date weekEnd = (Date) getLastWeekRange().get("stopTime");
        Date monthStart = (Date) getLastMonthRange().get("startTime");
        Date monthEnd = (Date) getLastMonthRange().get("stopTime");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");


        JSONArray jsonArray = new JSONArray();
        for (EnergyEnvDevice envDevice : envDevices) {
            EnergyDataScreenVo dataScreenVo = dataMapper.listDataAll(sdf.format(timesmorning), sdf.format(dateTime), sdf.format(timesMonthmorning),
                    sdf.format(dayStart), sdf.format(dayEnd)
                    , sdf.format(weekStart), sdf.format(weekEnd)
                    , sdf.format(monthStart), sdf.format(monthEnd)
                    , envDevice.getUnitId(), farmId);

            if ("1".equals(type) && "1".equals(envDevice.getRemark())) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("deviceCode", envDevice.getDeviceCode());
                jsonObject.put("buildName", envDevice.getUnitName());
                jsonObject.put("todayRead", dataScreenVo.getWaterValueToday());
                jsonObject.put("todayConsume", dataScreenVo.getWaterValueTodayUsage());
                jsonObject.put("yesterdayConsume", dataScreenVo.getWaterValueYesterdayUsage());
                jsonObject.put("weekConsume", dataScreenVo.getWaterValueThisWeekUsage());
                jsonObject.put("lastWeekConsume", dataScreenVo.getWaterValueLastWeekUsage());
                jsonObject.put("monthConsume", dataScreenVo.getElectricityValueThisMonthUsage());
                jsonObject.put("lastMonthConsume", dataScreenVo.getElectricityValueLastMonthUsage());
                double day2 = Double.parseDouble(dataScreenVo.getWaterValueTodayUsage());
                double day3 = Double.parseDouble(dataScreenVo.getWaterValueYesterdayUsage());
                double week1 = Double.parseDouble(dataScreenVo.getWaterValueThisWeekUsage());
                double week2 = Double.parseDouble(dataScreenVo.getWaterValueLastWeekUsage());
                double month1 = Double.parseDouble(dataScreenVo.getElectricityValueThisMonthUsage());
                double month2 = Double.parseDouble(dataScreenVo.getElectricityValueLastMonthUsage());
                DecimalFormat def = new DecimalFormat("0.00");
                String day4 = def.format(day2 - day3);
                String week3 = def.format(week1 - week2);
                String month3 = def.format(month1 - month2);
                jsonObject.put("dayDifference", day4);
                jsonObject.put("weekDifference", week3);
                jsonObject.put("monthDifference", month3);
                jsonArray.add(jsonObject);
            }
            if ("2".equals(type) && "0".equals(envDevice.getRemark())) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("deviceCode", envDevice.getDeviceCode());
                jsonObject.put("buildName", envDevice.getUnitName());
                jsonObject.put("todayRead", dataScreenVo.getElectricityValueToday());
                jsonObject.put("todayConsume", dataScreenVo.getElectricityValueTodayUsage());
                jsonObject.put("yesterdayConsume", dataScreenVo.getElectricityValueYesterdayUsage());
                jsonObject.put("weekConsume", dataScreenVo.getElectricityValueThisWeekUsage());
                jsonObject.put("lastWeekConsume", dataScreenVo.getElectricityValueLastWeekUsage());
                jsonObject.put("monthConsume", dataScreenVo.getElectricityValueThisMonthUsage());
                jsonObject.put("lastMonthConsume", dataScreenVo.getElectricityValueLastMonthUsage());
                double day2 = Double.parseDouble(dataScreenVo.getElectricityValueTodayUsage());
                double day3 = Double.parseDouble(dataScreenVo.getElectricityValueYesterdayUsage());
                double week1 = Double.parseDouble(dataScreenVo.getElectricityValueThisWeekUsage());
                double week2 = Double.parseDouble(dataScreenVo.getElectricityValueLastWeekUsage());
                double month1 = Double.parseDouble(dataScreenVo.getElectricityValueThisMonthUsage());
                double month2 = Double.parseDouble(dataScreenVo.getElectricityValueLastMonthUsage());
                DecimalFormat def = new DecimalFormat("0.00");
                String day4 = def.format(day2 - day3);
                String week3 = def.format(week1 - week2);
                String month3 = def.format(month1 - month2);
                jsonObject.put("dayDifference", day4);
                jsonObject.put("weekDifference", week3);
                jsonObject.put("monthDifference", month3);
                jsonArray.add(jsonObject);
            }

        }
        return new Result(ResultCode.SUCCESS, jsonArray);
    }

    /**
     * 获取上周开始时间和结束时间
     */
    public static JSONObject getLastWeekRange() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        calendar.add(Calendar.DAY_OF_WEEK, -7);
        Date startTime = calendar.getTime();
        calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH), 23, 59, 59);
        calendar.add(Calendar.DAY_OF_WEEK, 6);
        Date stopTime = calendar.getTime();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("startTime", startTime);
        jsonObject.put("stopTime", stopTime);
        return jsonObject;
    }

    /**
     * 获取上月开始时间和结束时间
     */
    public static JSONObject getLastMonthRange() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
        Date startTime = calendar.getTime();
        calendar.add(Calendar.DAY_OF_WEEK, 7);
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        //calendar.set(Calendar.HOUR_OF_DAY, 24);
        calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH), 23, 59, 59);
        Date stopTime = calendar.getTime();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("startTime", startTime);
        jsonObject.put("stopTime", stopTime);
        return jsonObject;
    }

    /**
     * 获取昨天开始时间和结束时间
     */
    public static JSONObject getYesterdayRange() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.HOUR_OF_DAY, -24);
        Date startTime = calendar.getTime();
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        Date stopTime = calendar.getTime();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("startTime", startTime);
        jsonObject.put("stopTime", stopTime);
        return jsonObject;
    }

    /*数组排序*/
    public static JSONArray jsonArraySort(JSONArray jsonArr, String sortKey, boolean is_desc) {
        //存放排序结果json数组
        JSONArray sortedJsonArray = new JSONArray();
        //用于排序的list
        List<JSONObject> jsonValues = new ArrayList<JSONObject>();
        //将参数json数组每一项取出，放入list
        if (jsonArr.size() < 8) {
            for (int i = 0; i < jsonArr.size(); i++) {
                jsonValues.add(jsonArr.getJSONObject(i));
            }
            //快速排序，重写compare方法，完成按指定字段比较，完成排序
            Collections.sort(jsonValues, new Comparator<JSONObject>() {
                //排序字段
                private final String KEY_NAME = sortKey;

                //重写compare方法
                @Override
                public int compare(JSONObject a, JSONObject b) {
                    //如果用String接会导致一位数和两位数混合比对的时候不能准确比出来，要用int类型接
                    //String valA = new String();
                    //String valB = new String();
                    Float valA = 0.00f;
                    Float valB = 0.00f;
                    try {
                        valA = Float.parseFloat(a.getString(KEY_NAME));
                        valB = Float.parseFloat(b.getString(KEY_NAME));
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    //是升序还是降序
                    if (is_desc) {
                        return -valA.compareTo(valB);
                    } else {
                        return -valB.compareTo(valA);
                    }

                }
            });
            //将排序后结果放入结果jsonArray
            for (int i = 0; i < jsonArr.size(); i++) {
                sortedJsonArray.add(jsonValues.get(i));
            }
        } else {
            for (int i = 0; i < jsonArr.size(); i++) {
                jsonValues.add(jsonArr.getJSONObject(i));
            }
            //快速排序，重写compare方法，完成按指定字段比较，完成排序
            Collections.sort(jsonValues, new Comparator<JSONObject>() {
                //排序字段
                private final String KEY_NAME = sortKey;

                //重写compare方法
                @Override
                public int compare(JSONObject a, JSONObject b) {
                    //如果用String接会导致一位数和两位数混合比对的时候不能准确比出来，要用int类型接
                    //String valA = new String();
                    //String valB = new String();
                    Float valA = 0.00f;
                    Float valB = 0.00f;
                    try {
                        valA = Float.parseFloat(a.getString(KEY_NAME));
                        valB = Float.parseFloat(b.getString(KEY_NAME));
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    //是升序还是降序
                    if (is_desc) {
                        return -valA.compareTo(valB);
                    } else {
                        return -valB.compareTo(valA);
                    }

                }
            });
            //将排序后结果放入结果jsonArray
            for (int i = 0; i < 8; i++) {
                sortedJsonArray.add(jsonValues.get(i));
            }
        }

        return sortedJsonArray;
    }

    /*数组排序*/
    public static JSONArray jsonArraySort1(JSONArray jsonArr, String sortKey, boolean is_desc) {
        //存放排序结果json数组
        JSONArray sortedJsonArray = new JSONArray();
        //用于排序的list
        List<JSONObject> jsonValues = new ArrayList<JSONObject>();
        //将参数json数组每一项取出，放入list

        for (int i = 0; i < jsonArr.size(); i++) {
            jsonValues.add(jsonArr.getJSONObject(i));
        }
        //快速排序，重写compare方法，完成按指定字段比较，完成排序
        Collections.sort(jsonValues, new Comparator<JSONObject>() {
            //排序字段
            private final String KEY_NAME = sortKey;

            //重写compare方法
            @Override
            public int compare(JSONObject a, JSONObject b) {
                //如果用String接会导致一位数和两位数混合比对的时候不能准确比出来，要用int类型接
                //String valA = new String();
                //String valB = new String();
                Float valA = 0.00f;
                Float valB = 0.00f;
                try {
                    valA = Float.parseFloat(a.getString(KEY_NAME));
                    valB = Float.parseFloat(b.getString(KEY_NAME));
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                //是升序还是降序
                if (is_desc) {
                    return -valA.compareTo(valB);
                } else {
                    return -valB.compareTo(valA);
                }

            }
        });
        //将排序后结果放入结果jsonArray
        if (jsonArr.size() < 5) {
            for (int i = 0; i < jsonArr.size(); i++) {
                sortedJsonArray.add(jsonValues.get(i));
            }
        } else {
            for (int i = 0; i < 5; i++) {
                sortedJsonArray.add(jsonValues.get(i));
            }
        }
        return sortedJsonArray;
    }

}
